﻿:Namespace ZipWeatherWS
⍝ === VARIABLES ===

    today←2012 10 8


⍝ === End of variables definition ===

    ⎕IO ⎕ML ⎕WX←1 0 3

    ∇ (rc wdata emsg)←WeatherData zip;host;port;page;resp;xml;latlon;lat;lon;fmtTS;args;data;ts;temp;dewp;rh;ws;val;wd;sky;appt
     ⍝ get weather forecast data for a zip code from the National Oceanographic and Atmospheric Administration (NOAA)
      (host port page)←'graphical.weather.gov' 80 '/xml/SOAP_server/ndfdXMLserver.php' ⍝ web service location
      emsg←''
      wdata←0 5⍴⊂⍬
      resp←(host port page)#.SAWS.Call'ndfdXML' 'LatLonListZipCode'('zipCodeList'zip) ⍝ look up the lat/long for the zip code
      :If 0=⊃(rc xml emsg)←extractXML resp'LatLonListZipCode' 'listLatLonOut' ⍝ grab the xml with the lat/long elements
          :If 0∊⍴latlon←xml getelement'latLonList' ⍝ grab the lat/long
              emsg←'Could not retrieve lat/lon'
          :Else
              lat lon←{1↓¨(⍵=⊃⍵)⊂⍵}',',latlon ⍝ split the lat and long
              fmtTS←{,'I4,<->,ZI2,<->,ZI2,<T>,ZI2,<:>,ZI2,<:>,ZI2'⎕FMT⍉⍪⍵}
              args←↑(1 'latitude'lat)(1 'longitude'lon)(1 'product' 'time-series')(1 'startTime'(fmtTS 6↑today←3↑⎕TS))
              args⍪←↑(1 'endTime'(fmtTS 23 59 59,⍨3↑2 ⎕NQ'.' 'IDNToDate'(7+2 ⎕NQ'.' 'DateToIDN'today)))(1 'unit' 'm')
              args⍪←(1 'weatherParameters' '')⍪1,⍨2,⍪'temp' 'dew' 'wspd' 'rh' 'wdir' 'sky' 'appt'
              data←(host port page)#.SAWS.Call'ndfdXML' 'NDFDgen'args
              val←{3⊃,⍵}
              :If 0=⊃(rc xml emsg)←extractXML data'NDFDgen' 'dwmlOut' ⍝ grab the xml from the forecast
                  ts←convertTS¨val¨'start-valid-time'gettag⍨⊃xml gettag'time-layout' 1
                  temp←⊃∘tonum¨val¨'value'gettag⍨⊃xml gettag'temperature' 1('type' 'hourly') ⍝ ○Celsius
                  dewp←⊃∘tonum¨val¨'value'gettag⍨⊃xml gettag'temperature' 1('type' 'dew point') ⍝ ○Celsius
                  rh←⊃∘tonum¨val¨'value'gettag⍨⊃xml gettag'humidity' 1  ⍝ %
                  ws←⊃∘tonum¨val¨'value'gettag⍨⊃xml gettag'wind-speed' 1 ⍝ meters/second
                  wd←⊃∘tonum¨val¨'value'gettag⍨⊃xml gettag'direction' 1 ⍝ wind direction
                  sky←⊃∘tonum¨val¨'value'gettag⍨⊃xml gettag'cloud-amount' 1 ⍝ sky cover %
                  appt←⊃∘tonum¨val¨'value'gettag⍨⊃xml gettag'temperature' 1('type' 'apparent') ⍝ apparent temp
                  wdata←ts,temp,dewp,appt,rh,sky,ws,[1.1]wd
              :EndIf
          :EndIf
      :EndIf
    ∇

    convertTS←{6↑⊃(//)⎕VFI ⍵{⍵\⍵/⍺}⍵∊⎕D}

    ∇ r←extractXML(data method result);emsg;xml;rc;xmlmat
                ⍝ extracts XML result from the output message for a method
      rc←1 ⋄ xmlmat←0 5⍴⊂⍬ ⋄ emsg←''
      :If 0≠⊃data
          emsg←'Invalid request'
      :ElseIf method≢2 1⊃data
          emsg←'Method not found'
      :ElseIf 0∊⍴xml←(2 2⊃data)getelement result
          emsg←'Result not found'
      :Else
          :Trap 0
              xmlmat←⎕XML xml
              rc←0 ⍝ success!
          :Else
              emsg←'Error pasring XML'
          :EndTrap
      :EndIf
      r←rc xmlmat emsg
    ∇

    getelement←{(⍺[;2]⍳⊂⍵)⊃⍺[;3],⊂''}

    ∇ r←xml gettag arg;⎕ML;element;kids;attrs;gotattrs;mask;kidmask
                ⍝ returns tag(s) matching arg
                ⍝ arg[1] - tag name to match
                ⍝ arg[2] - Boolean indicating whether to include child tags (default - 0)
                ⍝ arg[3] - attribute name/value pairs to match (default - none)
                ⍝ xml - 4 or 5 column ⎕XML matrix
                ⍝ r - vector of matching tags
      ⎕ML←1
      r←⍬
      :If 1=≡arg ⋄ arg←,⊂arg ⋄ :EndIf ⍝ only tag name supplied?
      element kids attrs←3↑arg,(⍴arg)↓'' 0 ''
      :If gotattrs←~0∊⍴attrs ⍝ if attrs is not empty
          :If (2=≡attrs)∧(1=⍴⍴attrs)∧0=2|¯1↑⍴attrs ⋄ attrs←((0.5××/⍴attrs),2)⍴attrs ⍝ attrs is a vector of name/value pairs
          :ElseIf 3=≡attrs ⋄ attrs←↑attrs ⍝ attrs is a vector of nested name/value pairs (('name1' 'value1')('name2' 'value2'))
          :EndIf
      :EndIf
      :If ∨/mask←xml[;2]≡¨⊂element ⍝ find matching tag names
          :If gotattrs ⋄ mask←mask\(⊂attrs){∧/∨⌿(4⊃⍵)∧.≡⍉⍺}¨↓mask⌿xml ⋄ :EndIf  ⍝ if attributes, match all supplied
          :If kids ⋄ mask←(mask/⍳⍴mask){(-⍴⍵)↑1,∧\(⍺⊃⍵)<⍺↓⍵}¨⊂xml[;1]
              r←mask⌿¨⊂xml
          :Else ⋄ r←1⊂[1]mask⌿xml
          :EndIf
      :EndIf
    ∇

    tonum←{⎕ML←1 ⋄ t←⍵ ⋄ z←(('-'=t)/t)←'¯' ⋄ ⊃(//)⎕VFI t}

:EndNamespace